!  BIOSData.f90 
! BIOS Data sample
! Copyright (c) 1999-2003, Zeal SoftStudio
! http://www.zealsoft.com/memaccess/
! support@zealsoft.com
! This sample demonstates how to use MemAccess Library to
! access the BIOS data
!
!  FUNCTIONS:
!   WinMain()      - Entry point for the application;
!                    displays the main window; processes the message loop
!   BIOSDataSub()  - Callback routine for the main dialog box
!   BIOSDataApply()- Callback routine for the APPLY button
!

!****************************************************************************
!
!  FUNCTION: WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!
!  PURPOSE:  Entry point for the application
!
!  COMMENTS: Displays the main window and processes the message loop
!
!****************************************************************************
include '../memacc.f90'

integer*4 function WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!DEC$ IF DEFINED(_X86_)
!DEC$ ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
!DEC$ ELSE
!DEC$ ATTRIBUTES STDCALL, ALIAS : 'WinMain' :: WinMain
!DEC$ ENDIF

    use user32
    use kernel32
    use dflogm
    use dfcom
    use dfauto
    use BIOSDataGlobals
	use memacc   !To use the MemAccess Library

    implicit none

    integer*4 hInstance
    integer*4 hPrevInstance
    integer*4 lpszCmdLine
    integer*4 nCmdShow

    include 'resource.fd'

    external BIOSDataSub
    external BIOSDataApply

    ! Variables
    type (T_MSG)            mesg
    integer*4               ret
    logical*4               lret


    ghInstance = hInstance
    ghModule = GetModuleHandle(NULL)
    ghwndMain = NULL
    call COMINITIALIZE(ret)

	! Place your registration information here
	call maLicenseInfo("Your Name", 0)

    lret = DlgInit(IDD_BIOSDATA_DIALOG, gdlg)
    if (lret == .FALSE.) goto 99999
    lret = DlgSetSub(gdlg, IDD_BIOSDATA_DIALOG, BIOSDataSub)
    lret = DlgSetSub(gdlg, IDM_APPLY, BIOSDataApply)
    lret = DlgModeless(gdlg, nCmdShow)
    if (lret == .FALSE.) goto 99999

    ! Read and process messsages
    do while( GetMessage (mesg, NULL, 0, 0) ) 
       if ( DlgIsDlgMessage(mesg) .EQV. .FALSE. ) then
           lret = TranslateMessage( mesg )
           ret  = DispatchMessage( mesg )
       end if
    end do
    call DlgUninit(gdlg)
    call COMUNINITIALIZE()

    WinMain = mesg.wParam
    return

99999 &

    ret = MessageBox(ghwndMain, "Error initializing application BIOSData"C, &
                     "Error"C, MB_OK)
    call COMUNINITIALIZE()
    WinMain = 0

end 

!****************************************************************************
!
!  FUNCTION: BIOSDataSub ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for initialization and destroy
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE BIOSDataSub( dlg, id, callbacktype )

  use user32
  use dflogm
  use dfcom
  use dfauto

  implicit none

  type (dialog) dlg
  integer id, callbacktype

  if (callbacktype == dlg_destroy) then
    call PostQuitMessage(0)
  endif

  END SUBROUTINE BIOSDataSub

!****************************************************************************
!
!  FUNCTION: BIOSDataApply ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for APPLY button
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE BIOSDataApply( dlg, id, callbacktype )

  use dflogm
  use dfcom
  use dfauto
  use user32
  use kernel32
  use memacc   !To use the MemAccess Library

  implicit none

  ENTRY GetBIOSDate

  include 'resource.fd'
  type (dialog) dlg
  integer id, callbacktype
  
  integer i, nAddress, b
  character (100) textstate, text
  character (10) textdate
  integer temp
  logical retlog 

  if (callbacktype == dlg_clicked) then
   
    ! Open the library
    if (maOpenLibrary() == .FALSE.) then
        temp = maGetLastState(textstate)
        temp = MessageBox(NULL , textstate, 'Error', MB_ICONSTOP)
        call exit
    endif
    
    retlog = DlgSet( dlg, IDC_LIST1, 7 )
    ! COM ports
    do i = 1, 4
        nAddress = maPeekW(Z'400' + (i - 1) * 2)
        if(maGetLastState(textstate) .NE. ERROR_SUCCESS) then
	        temp = MessageBox(NULL , textstate, 'Error', MB_ICONSTOP)
            call exit
        elseif(nAddress .NE. 0) then
            write(text, 500) i, nAddress
500         format(" COM",Z1,': ',Z8,'H')
	        temp = DlgSet( dlg, IDC_LIST1, text, i)
        else
			write(text, 600) i
600         format(" COM",Z1,' absent')
	        temp = DlgSet( dlg, IDC_LIST1, text, i)
        endif
    end do
    
    ! LPT ports
    do i = 1, 3
        nAddress = maPeekW(Z'408' + (i - 1) * 2)
        if(maGetLastState(textstate) .NE. ERROR_SUCCESS) then
	        temp = MessageBox(NULL , textstate, 'Error', MB_ICONSTOP)
            call exit
        elseif(nAddress .NE. 0) then
            write(text, 700) i, nAddress
700         format(" LPT",Z1,': ',Z8,'H')
	        temp = DlgSet( dlg, IDC_LIST1, text, i + 4)
        else
			write(text, 800) i
800         format(" LPT",Z1,' absent')
	        temp = DlgSet( dlg, IDC_LIST1, text, i + 4)
        endif
	end do
      
    textdate = ""
    do i = 1, 10
        b = maPeekB(V86_To_PhyAddress(Z'FFFF', 4) + i)
        if (b == 0) exit
        textdate(i:i) = char(b)
    end do

    write(text, *) "BIOS date:", textdate
	temp = DlgSet( dlg, IDC_STATIC1, text) 
    
    ! Close the library
    temp = maCloseLibrary()
  endif

  END SUBROUTINE BIOSDataApply


