!  MemTest.f90 
! MemTest sample
! Copyright (c) 1999-2003, Zeal SoftStudio
! http://www.zealsoftstudio.com
! support@zealsoft.com
! This sample demonstates how to use MemAccess Library
! to read 256 bytes.
!
!  FUNCTIONS:
!   WinMain()      - Entry point for the application;
!                    displays the main window; processes the message loop
!   MemTestSub()  - Callback routine for the main dialog box
!   MemTestApply()- Callback routine for the APPLY button
!

!****************************************************************************
!
!  FUNCTION: WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!
!  PURPOSE:  Entry point for the application
!
!  COMMENTS: Displays the main window and processes the message loop
!
!****************************************************************************
include "../memacc.f90"

integer*4 function WinMain( hInstance, hPrevInstance, lpszCmdLine, nCmdShow )
!DEC$ IF DEFINED(_X86_)
!DEC$ ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
!DEC$ ELSE
!DEC$ ATTRIBUTES STDCALL, ALIAS : 'WinMain' :: WinMain
!DEC$ ENDIF

    use user32
    use kernel32
    use dflogm
    use MemTestGlobals
	use memacc

    implicit none

    integer*4 hInstance
    integer*4 hPrevInstance
    integer*4 lpszCmdLine
    integer*4 nCmdShow

    include 'resource.fd'

    external MemTestSub
    external MemTestApply
    external MemTestWrite

    ! Variables
    type (T_MSG)            mesg
    integer*4               ret
    logical*4               lret


    ghInstance = hInstance
    ghModule = GetModuleHandle(NULL)
    ghwndMain = NULL

	! Place your registration information here
	call maLicenseInfo("Your Name", 0)

    lret = DlgInit(IDD_MEMTEST_DIALOG, gdlg)
    if (lret == .FALSE.) goto 99999
	lret = DlgSet(gdlg, IDC_EDIT1, "F8000")
    lret = DlgSetSub(gdlg, IDD_MEMTEST_DIALOG, MemTestSub)
    lret = DlgSetSub(gdlg, IDM_APPLY, MemTestApply)
    lret = DlgSetSub(gdlg, IDM_FILL, MemTestWrite)
    lret = DlgModeless(gdlg, nCmdShow)
    if (lret == .FALSE.) goto 99999

    ! Read and process messsages
    do while( GetMessage (mesg, NULL, 0, 0) ) 
       if ( DlgIsDlgMessage(mesg) .EQV. .FALSE. ) then
           lret = TranslateMessage( mesg )
           ret  = DispatchMessage( mesg )
       end if
    end do
    call DlgUninit(gdlg)

    WinMain = mesg.wParam
    return

99999 &

    ret = MessageBox(ghwndMain, "Error initializing application MemTest"C, &
                     "Error"C, MB_OK)
    WinMain = 0

end 

!****************************************************************************
!
!  FUNCTION: MemTestSub ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for initialization and destroy
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE MemTestSub( dlg, id, callbacktype )

  use user32
  use dflogm

  implicit none

  type (dialog) dlg
  integer id, callbacktype

  if (callbacktype == dlg_destroy) then
    call PostQuitMessage(0)
  endif

  END SUBROUTINE MemTestSub

!****************************************************************************
!
!  FUNCTION: MemTestApply ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for APPLY button
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE MemTestApply( dlg, id, callbacktype )

  use dflogm
  use memacc
  use user32

  implicit none

  include 'resource.fd'

  type (dialog) dlg
  integer id, callbacktype, i
  integer*4 j, dwAddr, temp
  character(100) textstate
  character(1000) textdata
  character(2) text
    

  if (callbacktype == dlg_clicked) then
    ! Open the library
    if (maOpenLibrary() == .FALSE.) then
        temp = maGetLastState(textstate)
        temp = MessageBox(NULL , textstate, 'Error', MB_ICONSTOP)
        call exit
    endif
    
    ! Get the physical address
	temp = DlgGet(dlg, IDC_EDIT1, textdata)
    if (LEN(textdata) .GT. 0) then
		read (textdata, 100) dwAddr
100     format(Z8)
	else 
        temp = MessageBox(NULL , "Please input a valid address!", &
					"Error", MB_ICONSTOP)
        call exit
	end if
   
    ! Read 256 bytes
    textdata = ""
	j = 1;
    do i = 0, 255
		write(text,200) maPeekB(dwAddr + i)
200     format(Z02)
        textdata(j : j + 1) = text
		if (textdata(j : j) == ' ') textdata(j : j) = '0'
		j = j + 3
        if (mod(i, 16) == 15) then
            textdata(j : j + 1) = char(13) // char(10)
			j = j + 2
        endif
    enddo
	
	temp = DlgSet(dlg, IDC_EDIT2, textdata)

	! Close the library
    call maCloseLibrary()
  endif

  END SUBROUTINE MemTestApply

!****************************************************************************
!
!  FUNCTION: MemTestWrite ( dlg, id, callbacktype )
!
!  PURPOSE:  Dialog box callback for Write button
!
!  COMMENTS:
!
!****************************************************************************

SUBROUTINE MemTestWrite( dlg, id, callbacktype )

  use dflogm
  use memacc
  use user32

  implicit none

  include 'resource.fd'

  type (dialog) dlg
  integer id, callbacktype, i
  integer*4 dwAddr, temp
  character(100) textstate, textdata
    

  if (callbacktype == dlg_clicked) then
    ! Open the library
    if (maOpenLibrary() == .FALSE.) then
        temp = maGetLastState(textstate)
        temp = MessageBox(NULL , textstate, 'Error', MB_ICONSTOP)
        call exit
    endif
    
    ! Get the physical address
	temp = DlgGet(dlg, IDC_EDIT1, textdata)
    if (LEN(textdata) .GT. 0) then
		read (textdata, 300) dwAddr
300     format(Z8)
	else 
        temp = MessageBox(NULL , "Please input a valid address!", &
					"Error", MB_ICONSTOP)
        call exit
	end if
   
    ! Write 256 bytes
    do i = 0, 255
		call maPokeB(dwAddr + i, i)
    enddo
	
	! Close the library
    call maCloseLibrary()
  endif

  END SUBROUTINE MemTestWrite

